<?php

namespace Sendeasy;

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit();
}

/**
 * SE_Sendeasy class.
 */
class SE_Sendeasy
{
    public function __construct()
    {
        add_action('plugins_loaded', [$this, 'se_init']);
        add_action('init', [$this, 'se_register_assets']);
        add_action('init', [$this, 'se_load_plugin_text_domain']);
        add_action('admin_enqueue_scripts', [$this, 'se_enqueue_admin_scripts']);
    }

    public function se_register_assets(): void
    {
        wp_register_script('se_admin', plugins_url('assets/js/admin.js', SE_SENDEASY_PLUGIN_FILE_PATH), [], '1.0.0', true);
    }

    public function se_enqueue_admin_scripts(string $hook_suffix): void
    {
        if ($hook_suffix === 'woocommerce_page_wc-settings') {
            wp_enqueue_script('se_admin');
        }
    }

    /**
     * Used to send a customer SMS notification
     *
     * @param int $order_id
     * @return void
     */
    public function se_send_customer_sms_notification_action(int $order_id): void
    {
        $username = get_option('se_username');
        $password = get_option('se_password');
        $enabled_sms_notifications = get_option('se_enabled_customer_sms_notifications');

        if (!empty($username) && !empty($password) && !empty($enabled_sms_notifications)) {

            $order = wc_get_order($order_id);

            $order_status = $order->get_status();

            if (in_array($order_status, $enabled_sms_notifications)) {

                $data = [
                    'fields' => [
                        'uID' => $username,
                        'pwd' => $password
                    ]
                ];

                $tags = [
                    '{shop_name}' => get_bloginfo(),
                    '{order_id}' => $order_id,
                    '{order_status}' => $order_status,
                    '{order_total}' => get_woocommerce_currency_symbol() . $order->get_total(),
                    '{billing_first_name}' => $order->get_billing_first_name(),
                    '{billing_last_name}' => $order->get_billing_last_name(),
                    '{shipping_first_name}' => $order->get_shipping_first_name(),
                    '{shipping_last_name}' => $order->get_shipping_last_name(),
                    '{shipping_method}' => $order->get_shipping_method()
                ];

                $message_template = get_option("se_customer_sms_template_{$order_status}");

                if (empty($message_template)) {
                    $message_template = get_option('se_customer_sms_template_default');
                }

                $data['fields']['message'] = !empty($message_template) ? str_replace(array_keys($tags), array_values($tags), $message_template) : '';

                $phone_number = str_replace(' ', '', $order->get_billing_phone());

                if (SE_Validator::se_validate_phone_number($phone_number)) {

                    $data['fields']['destination'] = $phone_number;
                } else {

                    $formatted_phone_number = SE_Formatter::se_format_phone_number($phone_number);

                    if (SE_Validator::se_validate_phone_number($formatted_phone_number)) {
                        $data['fields']['destination'] = $formatted_phone_number;
                    }
                }

                if (isset($data['fields']['destination'])) {

                    extract(SE_SOAP_Client::se_send_sms($data));

                    $order->add_order_note(sprintf(__('%s customer SMS notification sent to %s regarding %s order #%s. Result: %s. Response: %s.', 'sendeasy'), 'SendEasy', $data['fields']['destination'], $order_status, $order_id, $response_status, $response_message));
                } else {

                    $order->add_order_note(sprintf(__('%s customer SMS notification aborted for %s order #%s. Invalid phone number "%s".', 'sendeasy'), 'SendEasy', $order_status, $order_id, $phone_number));

                    $debug_log_enabled = get_option('se_enable_debug_log');

                    if (!empty($debug_log_enabled) && $debug_log_enabled === 'yes') {
                        wc_get_logger()->debug("SendEasy customer SMS notification aborted for {$order_status} order #{$order_id}. Invalid phone number \"{$phone_number}\".", ['source' => 'sendeasy-log']);
                    }
                }
            }
        }
    }

    /**
     * Used to send an admin SMS notification
     *
     * @param int $order_id
     * @return void
     */
    public function se_send_admin_sms_notification_action(int $order_id): void
    {
        $username = get_option('se_username');
        $password = get_option('se_password');
        $admin_sms_enabled = get_option('se_enable_admin_sms');
        $admin_phone_numbers = get_option('se_admin_phone_numbers');
        $debug_log_enabled = get_option('se_enable_debug_log');

        if (!empty($username) && !empty($password) && !empty($admin_phone_numbers) && ($admin_sms_enabled === 'yes')) {

            $order = wc_get_order($order_id);

            $data = [
                'fields' => [
                    'uID' => $username,
                    'pwd' => $password
                ]
            ];

            $tags = [
                '{shop_name}' => get_bloginfo(),
                '{order_id}' => $order_id,
                '{order_status}' => $order->get_status(),
                '{order_total}' => get_woocommerce_currency_symbol() . $order->get_total(),
                '{billing_first_name}' => $order->get_billing_first_name(),
                '{billing_last_name}' => $order->get_billing_last_name(),
                '{shipping_first_name}' => $order->get_shipping_first_name(),
                '{shipping_last_name}' => $order->get_shipping_last_name(),
                '{shipping_method}' => $order->get_shipping_method()
            ];

            $message_template = get_option('se_admin_sms_template');

            $data['fields']['message'] = !empty($message_template) ? str_replace(array_keys($tags), array_values($tags), $message_template) : '';

            $phone_numbers = explode(',', str_replace(' ', '', $admin_phone_numbers));

            $filtered_phone_numbers = array_filter($phone_numbers, fn ($number) => !empty($number));

            $valid_phone_numbers = [];

            foreach ($filtered_phone_numbers as $phone_number) {
                if (SE_Validator::se_validate_phone_number($phone_number)) {
                    $valid_phone_numbers[] = $phone_number;
                } else {
                    $formatted_phone_number = SE_Formatter::se_format_phone_number($phone_number);

                    if (SE_Validator::se_validate_phone_number($formatted_phone_number)) {
                        $valid_phone_numbers[] = $formatted_phone_number;
                    } else {

                        $order->add_order_note(sprintf(__('A %s admin SMS notification was aborted for new order #%s because of an incorrectly formatted phone number: "%s".', 'sendeasy'), 'SendEasy', $order_id, $phone_number));

                        if (!empty($debug_log_enabled) && $debug_log_enabled === 'yes') {
                            wc_get_logger()->debug("A SendEasy admin SMS notification was aborted for new order #{$order_id} because of an incorrectly formatted phone number: \"{$phone_number}\".", ['source' => 'sendeasy-log']);
                        }
                    }
                }
            }

            if (!empty($valid_phone_numbers)) {

                $result = [];

                if (count($valid_phone_numbers) > 1) {
                    $data['fields']['destination'] = $phone_numbers;
                    $result = SE_SOAP_Client::se_send_sms($data, true);
                } else {
                    $data['fields']['destination'] = $phone_numbers[0];
                    $result = SE_SOAP_Client::se_send_sms($data);
                }

                extract($result);

                $order->add_order_note(sprintf(__('%s admin SMS notification sent regarding new order #%s. Result: %s. Response: %s.', 'sendeasy'), 'SendEasy', $order_id, $response_status, $response_message));
            } else {
                $order->add_order_note(sprintf(__('No %s admin SMS notification regarding new order #%s was sent. No correctly formatted admin phone numbers were found.', 'sendeasy'), 'SendEasy', $order_id));

                if (!empty($debug_log_enabled) && $debug_log_enabled === 'yes') {
                    wc_get_logger()->debug("No SendEasy admin SMS notification regarding new order #{$order_id} was sent. No correctly formatted admin phone numbers were found.", ['source' => 'sendeasy-log']);
                }
            }
        }
    }

    /**
     * Adds the WooCommerce order statuses the can trigger SMS notifications.
     *
     * @access public
     * @return void
     */
    public function se_add_sms_notification_actions(): void
    {
        add_action('woocommerce_new_order', [$this, 'se_send_admin_sms_notification_action']);
        add_action('woocommerce_order_status_pending', [$this, 'se_send_customer_sms_notification_action']);
        add_action('woocommerce_order_status_completed', [$this, 'se_send_customer_sms_notification_action']);
        add_action('woocommerce_order_status_on-hold', [$this, 'se_send_customer_sms_notification_action']);
        add_action('woocommerce_order_status_processing', [$this, 'se_send_customer_sms_notification_action']);
        add_action('woocommerce_order_status_refunded', [$this, 'se_send_customer_sms_notification_action']);
        add_action('woocommerce_order_status_cancelled', [$this, 'se_send_customer_sms_notification_action']);
        add_action('woocommerce_order_status_failed', [$this, 'se_send_customer_sms_notification_action']);
    }

    /**
     * Adds the WooCommerce admin settings page for SendEasy.
     *
     * @access public
     * @param array $settings
     * @return array
     */
    public function se_add_settings_page(array $settings): array
    {
        $settings[] = include_once SE_SENDEASY_PLUGIN_DIR_PATH . '/includes/SE_Sendeasy_WC_Settings_Page.php';
        return $settings;
    }

    /**
     * Adds a notice saying the WooCommerce is not active.
     *
     * @access private
     * @return void
     */
    private function se_wc_required_notice(): void
    {
        printf(
            '<div class="notice notice-error"><p>%s</p></div>',
            __('SendEasy requires WooCommerce to be installed and active.', 'sendeasy')
        );
    }

    /**
     * Enables WooCommerce High-Performance Order Storage (HPOS) compatibility.
     *
     * @access public
     * @return void
     */
    public function se_enable_wc_hpos_compatibility(): void
    {
        if (class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class )) {
            \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', SE_SENDEASY_PLUGIN_FILE_PATH, true );
        }
    }

    /**
     * Adds the plugin text domain.
     *
     * @return void
     * @access public
     */
    public function se_load_plugin_text_domain(): void
    {
        load_plugin_textdomain('sendeasy', false, basename(SE_SENDEASY_PLUGIN_DIR_PATH) . '/languages');
    }

    /**
     * Initialises the SendEasy WooCommerce plugin.
     *
     * @access public
     * @return void
     */
    public function se_init()
    {
        if (class_exists('WooCommerce')) {
            add_action('before_woocommerce_init', [$this, 'se_enable_wc_hpos_compatibility']);
            add_filter('woocommerce_get_settings_pages', [$this, 'se_add_settings_page'], 15);
            $this->se_add_sms_notification_actions();
        } else {
            add_action('admin_notices', [$this, 'se_wc_required_notice']);
        }
    }
}
